#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <GL/glut.h>
#include <vector>
#include <assert.h>
#include "Vector.h"
#include "Utility.h"
#include "Constants.h"

using namespace std;

/* Structure definitions */

//very simple data structure to store 3D points
typedef struct Vector3 {
  float x;
  float y;
  float z;
  Vector3(float in_x, float in_y, float in_z) : x(in_x), y(in_y), z(in_z) {}
  Vector3() {}
} Vector3;

//data structure to store triangle - 
//note that v1, v2, and v3 are indexes into the vertex array
typedef struct Tri {
  int v1;
  int v2;
  int v3;
  Vector3 normal;
  Vector3 color;
  Tri(int in_v1, int in_v2, int in_v3) : v1(in_v1), v2(in_v2), v3(in_v3), normal(0, 1, 0){}
  Tri() : normal(0, 1, 0) {}
} Tri;

// global variables
int GW, GH; // global width and height of the windows
float ulAng = 0.0; // angle to rotate left upper arm by
float urAng = 0.0; // angle to rotate right upper arm by
float lfAng = 0; // angle to rotate left forearm by
float rfAng = 0; // angle to rotate right forearm by
float ltAng = 0; // angle to rotate left thigh by
float llAng = 0; // angle to rotate left leg by
float rtAng = 0; // angle to rotate right thigh by
float rlAng = 0; // angle to rotate right leg by
float luffyX = 0, luffyY = 0, luffyZ = 0; // luffy position
float ltAx = 0, ltAy = 0, ltAz = 1; // left thigh axis of rotation
float rtAx = 0, rtAy = 0, rtAz = 1; // right thigh axis of rotation
float lfAx = 0, lfAy = 0, lfAz = 1; // left forearm axis of rotation
float rfAx = 0, rfAy = 0, rfAz = 1; // right forearm axis of rotation
float Lx = 0, Ly = 2, Lz = -20; // light vector components
//STL vector to store all the triangles in the mesh
vector<Tri *> Triangles;
//STL vector to store all the vertices in the mesh
vector<Vector3 *> Vertices;
//for computing the center point and extent of the model
Vector3 center;
float max_x, max_y, max_z, min_x, min_y, min_z;
float max_extent;
GLuint		shaderTexture[1];
Vector lightAngle;
float		outlineColor[3]	= { 0.0f, 0.0f, 0.0f };				// Color Of The Lines ( NEW )
float		outlineWidth	= 2.0f;								// Width Of The Lines ( NEW )
GLfloat objectM[16] = {1, 0, 0, 0,
                       0, 1, 0, 0,
                       0, 0, 1, 0,
                       0, 0, 0, 1};
GLfloat *M = (GLfloat *)objectM;

// global flags
int mode = NOTHING;
bool isRight = true; // either to continue waving to the user's right or not
bool isFall = false;
bool isForward = true;
bool isAttacking = true;
bool isFlapping = true;

// forward declarations
void readStream(istream& is);
void readLine(char* str);
void drawSphere(double r, int lats, int longs);
void wave();
void jump();
void walk();
void tiger();
void chickenDance();

//open the file for reading
void ReadFile(char* filename) 
{
  printf("Reading coordinates from %s\n", filename);
  
  ifstream in_f(filename);
  if (!in_f)  {
    printf("Could not open file %s\n", filename);
  } else {
    readStream(in_f);
  }
  
}

//process the input stream from the file
void readStream(istream& is)
{
  char str[256];
  for (;is;) {
    is >> ws;
    is.get(str,sizeof(str));
    if (!is) break;
    is.ignore(9999,'\n');
    readLine(str);
  }
}

//process each line of input save vertices and faces appropriately
void readLine(char* str) {
  
  int indx = 0, vi;
  float x, y, z;
  float r, g, b;
  int mat;
  
  if (str[0]=='#') return;
  //read a vertex or face
  if (str[0]=='V' && !strncmp(str,"Vertex ",7)) {
    Vector3* v;
    assert(sscanf(str,"Vertex %d %g %g %g",&vi,&x,&y,&z)==4);
    v = new Vector3(x, y, z);
    //store the vertex
    Vertices.push_back(v);
    //house keeping to display in center of the scene
    center.x += v->x;
    center.y += v->y;
    center.z += v->z;
    if (v->x > max_x) max_x = v->x; if (v->x < min_x) min_x = v->x;
    if (v->y > max_y) max_y = v->y; if (v->y < min_y) min_y = v->y;
    if (v->z > max_z) max_z = v->z; if (v->z < min_z) min_z = v->z;
  } else if (str[0]=='F' && !strncmp(str,"Face ",5)) {
    Tri* t;
    t = new Tri();
    char* s=str+4;
    int fi=-1;
    for (int t_i = 0;;t_i++) {
      while (*s && isspace(*s)) s++;
      //if we reach the end of the line break out of the loop
      if (!*s) break;
      //save the position of the current character
      char* beg=s;
      //advance to next space
      while (*s && isdigit(*s)) s++;
      //covert the character to an integer
      int j=atoi(beg);
      //the first number we encounter will be the face index, don't store it
      if (fi<0) { fi=j; continue; }
      //otherwise process the digit we've grabbed in j as a vertex index
      //the first number will be the face id the following are vertex ids
      if (t_i == 1)
        t->v1 = j;
      else if (t_i == 2)
        t->v2 = j;
      else if (t_i == 3)
        t->v3 = j;
      //if there is more data to process break out
      if (*s =='{') break;
    }
    //possibly process colors if the mesh has colors
    if (*s && *s =='{'){
      char *s1 = s+1;
      cout << "trying to parse color " << !strncmp(s1,"rgb",3) << endl;
      //if we're reading off a color
      if (!strncmp(s1,"rgb=",4)) {
        //grab the values of the string
        assert(sscanf(s1,"rgb=(%g %g %g) matid=%d",&r,&g,&b,&mat)==4);
        t->color.x = r; t->color.x = g; t->color.x = b;
        cout << "set color to: " << r << " " << g << " " << b << endl;
      }
    }
    //store the triangle read in
    Triangles.push_back(t);
  }
}

//testing routine for parser - left in just as informative code about accessing data
void printFirstThree() {
  printf("first vertex: %f %f %f \n", Vertices[0]->x, Vertices[0]->y, Vertices[0]->z);
  printf("first face: %d %d %d \n", Triangles[0]->v1, Triangles[0]->v2, Triangles[0]->v3);    
  printf("second vertex: %f %f %f \n", Vertices[1]->x, Vertices[1]->y, Vertices[1]);
  printf("second face: %d %d %d \n", Triangles[1]->v1, Triangles[1]->v2, Triangles[1]->v3);
  printf("third vertex: %f %f %f \n", Vertices[2]->x, Vertices[2]->y, Vertices[2]->z);
  printf("third face: %d %d %d \n", Triangles[2]->v1, Triangles[2]->v2, Triangles[2]->v3);
}

/* Draws the axes of the system in black.
 *
 * WARNING: This function modifies the color status to black.  You may need to 
 * change the color after this function call.
 */
void drawAxes()
{
	glPushMatrix();
    glColor3f(0,0,0);

    glBegin(GL_LINES);
        // Draw x axis
        glVertex3f(-2*(float)GW/GH*15, 0, -15);
        glVertex3f((float)2*GW/GH*15, 0, -15);

        // Draw y axis
        glVertex3f(0, -2*15, -15);
        glVertex3f(0, 2*15, -15);

        // Draw z axis
        glVertex3f(0, 0, -15);
        glVertex3f(0, 0, 15);
    glEnd();
	glPopMatrix();
}

//drawing routine to draw triangles as wireframe
void drawTria(Tri* t) {
    glBegin(GL_TRIANGLES);
    glColor3f(0.0, 0.0, 0.5);
    //note that the vertices are indexed starting at 0, but the triangles
    //index them starting from 1, so we must offset by -1!!!
    glVertex3f(Vertices[t->v1 - 1]->x, 
      Vertices[t->v1 - 1]->y,
      Vertices[t->v1 - 1]->z);
    glVertex3f(Vertices[t->v2 - 1]->x, 
      Vertices[t->v2 - 1]->y,
      Vertices[t->v2 - 1]->z);
    glVertex3f(Vertices[t->v3 - 1]->x, 
      Vertices[t->v3 - 1]->y,
      Vertices[t->v3 - 1]->z);
    glEnd();
}

// Engine Functions
bool Initialize ()					// Any GL Init Code & User Initialiazation Goes Here
{
	int i;														// Looping Variable ( NEW )
	char Line[255];												// Storage For 255 Characters ( NEW )
	float shaderData[32][3];									// Storate For The 96 Shader Values ( NEW )

	FILE *In	= NULL;											// File Pointer ( NEW )

	// Start Of User Initialization
	glHint (GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);			// Realy Nice perspective calculations
	
	glClearColor (0.7f, 0.7f, 0.7f, 0.0f);						// Light Grey Background
	glClearDepth (1.0f);										// Depth Buffer Setup

	glEnable (GL_DEPTH_TEST);									// Enable Depth Testing
	glDepthFunc (GL_LESS);										// The Type Of Depth Test To Do

	glShadeModel (GL_SMOOTH);									// Enables Smooth Color Shading ( NEW )
	glDisable (GL_LINE_SMOOTH);									// Initially Disable Line Smoothing ( NEW )

	glEnable (GL_CULL_FACE);									// Enable OpenGL Face Culling ( NEW )

	glDisable (GL_LIGHTING);									// Disable OpenGL Lighting ( NEW )

	In = fopen ("Data\\Shader.txt", "r");						// Open The Shader File ( NEW )

	if (In)														// Check To See If The File Opened ( NEW )
	{
		for (i = 0; i < 32; i++)								// Loop Though The 32 Greyscale Values ( NEW )
		{
			if (feof (In))										// Check For The End Of The File ( NEW )
				break;

			fgets (Line, 255, In);								// Get The Current Line ( NEW )

			shaderData[i][0] = shaderData[i][1] = shaderData[i][2] = float(atof (Line)); // Copy Over The Value ( NEW )
		}

		fclose (In);											// Close The File ( NEW )
	}

	else
		return false;											// It Went Horribly Horribly Wrong ( NEW )

	glGenTextures (1, &shaderTexture[0]);						// Get A Free Texture ID ( NEW )

	glBindTexture (GL_TEXTURE_1D, shaderTexture[0]);			// Bind This Texture. From Now On It Will Be 1D ( NEW )

	// For Crying Out Loud Don't Let OpenGL Use Bi/Trilinear Filtering! ( NEW )
	glTexParameteri (GL_TEXTURE_1D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);	
	glTexParameteri (GL_TEXTURE_1D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

    glTexImage1D (GL_TEXTURE_1D, 0, GL_RGB, 32, 0, GL_RGB , GL_FLOAT, shaderData);	// Upload ( NEW )

    lightAngle = Vector(Lx, Ly, Lz);
    lightAngle.normalize();										// Normalize The Light Direction ( NEW )
    
    return true;
}

void Deinitialize (void)										// Any User DeInitialization Goes Here
{
	glDeleteTextures (1, &shaderTexture[0]);					// Delete The Shader Texture ( NEW )
}

void drawLeftUpperArm()
{
    glPushMatrix();
        glTranslatef(-0.75, 0.0, 0.0);
        glScalef(0.75, 0.25, 0.25);
        drawSphere(1.0, 20, 20);
    glPopMatrix();
}

void drawRightUpperArm()
{
    glPushMatrix();
        glTranslatef(0.75, 0.0, 0.0);
        glScalef(0.75, 0.25, 0.25);
        drawSphere(1.0, 20, 20);
    glPopMatrix();
}

void RotateVector (GLfloat *M, Vector &V, Vector &D)				// Rotate A Vector Using The Supplied Matrix ( NEW )
{
	D.x = (M[0] * V.x) + (M[4] * V.y) + (M[8]  * V.z);	// Rotate Around The X Axis ( NEW )
	D.y = (M[1] * V.x) + (M[5] * V.y) + (M[9]  * V.z);	// Rotate Around The Y Axis ( NEW )
	D.z = (M[2] * V.x) + (M[6] * V.y) + (M[10] * V.z);	// Rotate Around The Z Axis ( NEW )
}

void doOutline(double r, int lats, int longs)
{
    double x, y, z = 0; // x, y, z coordinates for vertices

    glEnable (GL_BLEND);									// Enable Blending ( NEW )
    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);		// Set The Blend Mode ( NEW )

    glPolygonMode (GL_BACK, GL_LINE);						// Draw Backfacing Polygons As Wireframes ( NEW )
    glLineWidth (outlineWidth);								// Set The Line Width ( NEW )

    glCullFace (GL_FRONT);									// Don't Draw Any Front-Facing Polygons ( NEW )

    glDepthFunc (GL_LEQUAL);								// Change The Depth Mode ( NEW )

    glColor3fv (&outlineColor[0]);							// Set The Outline Color ( NEW )

    glBegin(GL_LINE_STRIP);
    for (double angle = 1.0/PI; angle <= 2.0*PI+1.0/PI; angle += 2.0*PI/lats) {
        x = r*sin(angle);
        y = r*cos(angle);
        glVertex3f(x, y, z);
    }
    glEnd();

    glDepthFunc (GL_LESS);									// Reset The Depth-Testing Mode ( NEW )

    glCullFace (GL_BACK);									// Reset The Face To Be Culled ( NEW )

    glPolygonMode (GL_BACK, GL_FILL);						// Reset Back-Facing Polygon Drawing Mode ( NEW )

    glDisable (GL_BLEND);									// Disable Blending ( NEW )

}

/**
 * Draws a cell-shaded sphere at the origin with radius r and lat number of 
 * latitudes and longs number of longitude lines.
 * 
 * WARNING: This sphere algorithm was taken from someone else -- apparently
 * they never use the passed in radius.  Which means that r is always = 1.
 */
void drawSphere(double r, int lats, int longs) {
    float TmpShade;
    Vector TmpVector, TmpNormal;

    // Cel-Shading Code //
    glEnable (GL_TEXTURE_1D);									// Enable 1D Texturing ( NEW )
    glBindTexture (GL_TEXTURE_1D, shaderTexture[0]);			// Bind Our Texture ( NEW )

    int i, j;
    for(i = 0; i <= lats; i++) {
        double lat0 = PI * (-0.5 + (double) (i - 1) / lats);
        double z0  = sin(lat0);
        double zr0 =  cos(lat0);

        double lat1 = PI * (-0.5 + (double) i / lats);
        double z1 = sin(lat1);
        double zr1 = cos(lat1);

        glBegin(GL_QUAD_STRIP);
        for(j = 0; j <= longs; j++) {
            double lng = 2 * PI * (double) (j - 1) / longs;
            double x = cos(lng);
            double y = sin(lng);

            TmpNormal = Vector(x * zr0, y * zr0, z0);
            RotateVector (M, TmpNormal, TmpVector);	// Rotate This By The Matrix ( NEW )

            TmpShade = Vector::dotProduct (TmpVector, lightAngle);	// Calculate The Shade Value ( NEW )

            if (TmpShade < 0.0f)
                TmpShade = 0.0f;							// Clamp The Value to 0 If Negative ( NEW )

            glTexCoord1f (TmpShade);						// Set The Texture Co-ordinate As The Shade Value ( NEW )

            //glNormal3f(x * zr0, y * zr0, z0);
            glVertex3f(x * zr0, y * zr0, z0);

            /* Next one... */

            TmpNormal = Vector(x * zr1, y * zr1, z1);
            RotateVector (M, TmpNormal, TmpVector);	// Rotate This By The Matrix ( NEW )

            TmpShade = Vector::dotProduct (TmpVector, lightAngle);	// Calculate The Shade Value ( NEW )

            if (TmpShade < 0.0f)
                TmpShade = 0.0f;							// Clamp The Value to 0 If Negative ( NEW )

            glTexCoord1f (TmpShade);						// Set The Texture Co-ordinate As The Shade Value ( NEW )
            //glNormal3f(x * zr1, y * zr1, z1);
            glVertex3f(x * zr1, y * zr1, z1);
        }
        glEnd();
    }
    glDisable (GL_TEXTURE_1D);

    doOutline(r, lats, longs);
}

/**
 * Draws a thigh with the axis of rotation at the origin.  The thigh drawn is 
 * nothing more than a scaled and translated sphere.
 */
void drawThigh()
{
    glPushMatrix();
        glTranslatef(0, -(thighHt), 0);
        glScalef(thighW, thighHt, 0.4);
        drawSphere(1.0, 20, 20);
    glPopMatrix();
}

/**
 * Draws a leg with the axis of rotation at the origin.  The leg drawn is 
 * nothing more than a scaled and translated sphere.
 */
void drawLeg()
{
    glPushMatrix();
        glTranslatef(0, -legHt, 0);
        glScalef(legW, legHt, legDepth);
        drawSphere(1.0, 20, 20);
    glPopMatrix();
}

/**
 * Draws a foot with the axis of rotation at the origin.
 */
void drawFoot()
{
    glPushMatrix();
        glTranslatef(0, 0, footDepth);
        glScalef(footW, footHt, footDepth);
        drawSphere(1, 20, 20);
    glPopMatrix();
}

/**
 * Draw an eye with its center at the origin.
 */
void drawEye()
{
    glPushMatrix();
        glScalef(eyeW, eyeHt, eyeDepth);
        drawSphere(1, 20, 20);
    glPopMatrix();
}

/**
 * Draws an eyeball with its center at the origin.
 */
void drawEyeball()
{
    glColor3fv(black);
    glPushMatrix();
        glPointSize(0.013*GH);
        glBegin(GL_POINTS);
            glVertex3f(-0.023, 0, 0);
        glEnd();
    glPopMatrix();
}

/**
 * Draws the mouthline.
 */
void drawMouthLine()
{
    glColor3fv(black);
    glBegin(GL_LINE_STRIP);
        glVertex3f(-0.37, 0.01, 0.01);
        glVertex3f(0, -0.01, 0);
        glVertex3f(0.37, 0.01, 0.01);
    glEnd();
}

/**
 * Draws a eyebrow above the origin.  The eyebrow is drawn is merely a sector
 * of a circle with a radius a little greater than the eye.
 */
void drawEyebrow()
{    
    float x, y; // x y coordinates
    float r; // radius
    const float nEyebrowSectors = 10;

    glLineWidth(0.0024*GH);
    glBegin(GL_LINE_STRIP);
    r = eyeW*1.5;
    for (int t = 135; t >= 45; t -= 135/nEyebrowSectors) {
        x = r*myCosf(t);
        y = r*mySinf(t);
        glVertex3f(x, y, 0);
    }
    glEnd();
}

/**
 * Draw hair on the face.  Uses headW, headHt, and headDepth to do its
 * computation.
 */
void drawHair()
{
    glPushMatrix();
        glColor3fv(black);
        glBegin(GL_QUAD_STRIP);
            glVertex3f(-0.6, 0.12, 0); // v0
            glVertex3f(-0.65, -0.7, 0); // v1
            glVertex3f(-0.65, 0.55, 0); // v2
            glVertex3f(-0.9, 0.7, 0); // v3
            glVertex3f(-0.3, 0.7, 0); // v4
            glVertex3f(-0.2, 0.85, 0); // v5
            glVertex3f(-0.2, 0.75, 0); // v6
            glVertex3f(-0.1, 0.9, 0); // v7
            glVertex3f(-0.05, 0.75, 0); // v8
            glVertex3f(0.08, 0.83, 0); // v9
            glVertex3f(0.04, 0.72, 0); // v10
            glVertex3f(0.2, 0.7, 0); // v11
            glVertex3f(0.2, 0.75, 0); // v12
            glVertex3f(0.31, 0.62, 0); // v13
            glVertex3f(0.45, 0.58, 0); // v14
            glVertex3f(0.4, 0.55, 0); // v15
            glVertex3f(0.61, 0.45, 0); // v16
            glVertex3f(0.55, 0.4, 0); //v17
            glVertex3f(0.71, 0.34, 0); //v18
            glVertex3f(0.6, -0.08, 0); //v19
            glVertex3f(0, 0.5, 0); // v20
            glVertex3f(-0.6, 0.12, 0); // v0
        glEnd();
    glPopMatrix();
}

/**
 * Draws the face with the axis of rotation at the origin.
 */
void drawFace()
{
    glPushMatrix();
        glTranslatef(0, headHt, 0); // translate to proper axis of rotation

        //drawHair();

        // draw head
        glColor3fv(skinColor);
        glPushMatrix();
            glScalef(headW, headHt, headDepth);
            drawSphere(1, 20, 20);
        glPopMatrix();

        // draw nose
        glLineWidth(0.0024*GH);
        glBegin(GL_LINE_STRIP);
            glVertex3f(0, 0.15, 0);
            glVertex3f(0.05, 0, 0);
            glVertex3f(0, 0, 0);
        glEnd();

        // draw mouth
        glPushMatrix();
            glTranslatef(0, -0.3, 0);

            drawMouthLine();

            glScalef(mouthW, mouthHt, mouthDepth);
            glColor3fv(white);
            drawSphere(1, 20, 20);
        glPopMatrix();

        // draw left eye
        glColor3fv(white);
        glPushMatrix();
            glTranslatef(-0.22, 0.2, 0);
            drawEye();
            drawEyeball();
            drawEyebrow();
        glPopMatrix();

        // draw right eye
        glColor3fv(white);
        glPushMatrix();
            glTranslatef(0.22, 0.2, 0);
            drawEye();
            drawEyeball();
            drawEyebrow();
        glPopMatrix();
    glPopMatrix();
}

/* Draws a Luffy model onto the screen at the origin in world coordinates.
 */
void drawLuffy()
{
	float x, y, z; // x y z values for width, height, and depth

    // Draw head
    glColor3fv(skinColor);
    glPushMatrix();
        glTranslatef(0, torsoHt+2*bodyHt+2*neckHt, 0);
        drawFace();
    glPopMatrix();

    // Draw neck
    glColor3fv(skinColor);
    glPushMatrix();
        glTranslatef(0.0, torsoHt+bodyHt*2.0, 0.0);
        glTranslatef(0, neckHt, 0); // translate neck to proper axis of rotation
        glScalef(neckW, neckHt, neckDepth);
        drawSphere(1, 20, 20);
    glPopMatrix();

    // Draw body
    glColor3f(0.8, 0.0, 0.1);
    glPushMatrix();
        glTranslatef(0, torsoHt, 0);
        glTranslatef(0, bodyHt, 0);
        glScalef(bodyW, bodyHt, bodyDepth);
        drawSphere(1, 20, 20);
    glPopMatrix();

    // Draw left arm
    glColor3fv(skinColor);
    glPushMatrix();
		// Draw left upper arm
        glTranslatef(-bodyW+0.2, bodyHt*2.0-0.2+torsoHt, 0.0);
		glRotatef(ulAng, 0.0, 0.0, 1.0);
        drawLeftUpperArm();

		// Draw left forearm
		x = uArmW+0.75;
		y = uArmHt/2.0;
		z = 0;

		// restrain left forearm angles to reasonable values
		if (lfAng > 80) {
			lfAng = 80;
		}
		else if (lfAng < -60) {
			lfAng = -60;
		}

		glColor3fv(skinColor);
		glPushMatrix();
			glTranslatef(-x, y, z);
			glRotatef(lfAng, lfAx, lfAy, lfAz);
			glTranslatef(0, 0.3, 0);
			glScalef(0.175, 0.6, 0.175);
			drawSphere(1, 20, 20);

			// Draw left hand
			//glColor3f(1,1,0);
            glColor3fv(skinColor);
			glPushMatrix();
				glTranslatef(0, 1, 0);
				glScalef(1.4, 0.4, 0.4);
				drawSphere(1, 20, 20);
			glPopMatrix();
		glPopMatrix();
	glPopMatrix();

    // Draw right arm
	glColor3fv(skinColor);
    glPushMatrix();
        glTranslatef(bodyW-0.2, bodyHt*2.0-0.2+torsoHt, 0.0);
        glRotatef(urAng, 0.0, 0.0, 1.0);
        drawRightUpperArm();

		// Draw right forearm
		x = uArmW+0.75;
		y = uArmHt/2.0;
		z = 0;
		
		// restrain left forearm angles to reasonable values
		if (rfAng > 60) {
			rfAng = 60;
		}
		else if (rfAng < -80) {
			rfAng = -80;
		}
		
		glPushMatrix();
			glTranslatef(x, y, z);
			glRotatef(rfAng, rfAx, rfAy, rfAz);
			glTranslatef(0, 0.3, 0);
			glScalef(0.175, 0.6, 0.175);
            glColor3fv(skinColor);
			drawSphere(1, 20, 20);

			// Draw right hand
			//glColor3f(1,1,0);
			glPushMatrix();
				glTranslatef(0, 1, 0);
				glScalef(1.4, 0.4, 0.4);
                glColor3fv(skinColor);
				drawSphere(1, 20, 20);
			glPopMatrix();
		glPopMatrix();
    glPopMatrix();

    // Draw lower torso
    glColor3f(0.0, 0.0, 0.8);
    glPushMatrix();
        glScalef(torsoW, torsoHt, 1.0);
        drawSphere(1.0, 20, 20);
    glPopMatrix();

    // Draw left thigh
    glColor3f(0.0, 0.0, 0.8);
    glPushMatrix();
    glTranslatef(-torsoW/1.2, -(torsoHt/2.0), 0.0);
    glRotatef(ltAng, ltAx, ltAy, ltAz);
    drawThigh();

        // Draw left leg
        glColor3fv(skinColor);
        glPushMatrix();
            glTranslatef(0, -(thighHt*2.0-0.1), 0);
            glRotatef(llAng, 0, 0, 1);
            drawLeg();

            // Draw left foot
            glColor3fv(skinColor);
            glPushMatrix();
                glTranslatef(0, -(legHt*2.0), 0);
                drawFoot();
            glPopMatrix();
        glPopMatrix();
    glPopMatrix();

    // Draw right thigh
    glColor3f(0.0, 0.0, 0.8);
    glPushMatrix();
        glTranslatef(torsoW/1.2, -(torsoHt/2.0), 0.0);
        glRotatef(rtAng, rtAx, rtAy, rtAz);
        drawThigh();

        // Draw right leg
        glColor3fv(skinColor);
        glPushMatrix();
            glTranslatef(0, -(thighHt*2.0-0.1), 0);
            glRotatef(rlAng, 0, 0, 1);
            drawLeg();

            // Draw right foot
            glColor3fv(skinColor);
            glPushMatrix();
                glTranslatef(0, -(legHt*2.0), 0);
                drawFoot();
            glPopMatrix();
        glPopMatrix();
    glPopMatrix();
}

void display()
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    gluLookAt(-2.0, 1.1, 7.0,  // eye position
              0.0, 1.1, -1.0, // Look at position
              0.0, 1.0, 0.0); // up vector

    glPushMatrix();
        glTranslatef(2, 2, 2);
        glScalef(10,10,10);
        for(int j = 0; j < Triangles.size(); j++) {
            drawTria(Triangles[j]);
        }
    glPopMatrix();
    //drawAxes();
    glTranslatef(luffyX, luffyY, luffyZ);
    drawLuffy();

    glutSwapBuffers();
}

void idle()
{
    switch (mode) {
        case WAVE:
            wave();
            break;
        case JUMP:
            jump();
            break;
        case WALK:
            walk();
            break;
        case TIGER:
            tiger();
            break;
        case CHICKEN:
            chickenDance();
            break;
    }

    glutPostRedisplay();
}

void keyboard(unsigned char key, int x, int y )
{
	const float diffAng = 2.0;

    switch(key) {
        case 'q': // move left upper arm clockwise
            ulAng -= diffAng;
            break;
        case 'Q': // move left upper arm CCW
            ulAng += diffAng;
            break;
		case 'w': // move left forearm CW
			lfAng -= diffAng;
			break;
		case 'W': // move left forearm CCW
			lfAng += diffAng;
			break;
        case 'e': // move left thigh CW
            ltAng -= diffAng;
            break;
        case 'E': // move left thigh CCW
            ltAng += diffAng;
            break;
        case 'r': // move left leg CW
            llAng += diffAng;
            break;
        case 'R': // move left leg CCW
            llAng -= diffAng;
            break;
        case 'a': // move right upper arm clockwise
            urAng -= diffAng;
            break;
        case 'A': // move right upper arm CCW
            urAng += diffAng;
            break;
		case 's': // move right forearm CW
			rfAng -= diffAng;
			break;
		case 'S': // move left forearm CCW
			rfAng += diffAng;
			break;
        case 'd': //move right thigh CW
            rtAng += diffAng;
            break;
        case 'D': // move right thigh CCW
            rtAng -= diffAng;
            break;
        case 'f': // move right leg CW
            rlAng += diffAng;
            break;
        case 'F': //move right leg CCW
            rlAng -= diffAng;
            break;
        case 'n': case 'N': // reset settings
            Lx = 0;
            Ly = 2;
            Lz = -20;
            lightAngle = Vector(Lx, Ly, Lz);
            lightAngle.normalize();

            ulAng = 0.0; // angle to rotate left upper arm by
            urAng = 0.0; // angle to rotate right upper arm by
            lfAng = 0; // angle to rotate left forearm by
            rfAng = 0; // angle to rotate right forearm by
            ltAng = 0; // angle to rotate left thigh by
            llAng = 0; // angle to rotate left leg by
            rtAng = 0; // angle to rotate right thigh by
            rlAng = 0; // angle to rotate right leg by
            luffyX = 0, luffyY = 0, luffyZ = 0; // luffy position
            ltAx = ltAy = lfAx = lfAy = 0;
            ltAz = lfAz= 1; // left thigh axis of rotation
            rtAx = rtAy = rfAx = rfAy = 0;
            rtAz = rfAz = 1; // right thigh axis of rotation

            // Initialize global flags
            mode = NOTHING;
            isRight = true; // either to continue waving to the user's right or not
            isFall = false;
            isForward = true;
            isAttacking = true;
            isFlapping = true;
            break;
        case 27: // 'Esc' key
            exit(EXIT_SUCCESS);
    }

    if (ulAng > 30) {
        ulAng = 30;
    }
    else if (ulAng < -70) {
        ulAng = -70;
    }

    if (urAng > 80) {
        urAng = 80;
    }
    else if (urAng < -35) {
        urAng = -35;
    }

    glutPostRedisplay();
}

/**
 * Process the special directional keys (left, right, down, up, page down/up)
 * used to position the light source.  Each direction key moves the light
 * source in that direction (page up moves the light source away).
 */
void processSpecialKeys(int key, int x, int y) {

	switch(key) {
		case GLUT_KEY_LEFT : 
			Lx -= 1;
            break;
		case GLUT_KEY_RIGHT : 
			Lx += 1;
            break;
		case GLUT_KEY_UP :
			Ly += 1;
            break;
        case GLUT_KEY_DOWN:
            Ly -= 1;
            break;
        case GLUT_KEY_PAGE_UP:
            Lz -= 1;
            break;
        case GLUT_KEY_PAGE_DOWN:
            Lz += 1;
            break;
	}

    lightAngle = Vector(Lx, Ly, Lz);
    lightAngle.normalize();
    glutPostRedisplay();
}

void reshape(int w, int h) 
{
    GW = w;
    GH = h;

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    //gluPerspective(90, (float)w/h, 1.0, 50.0);
    glOrtho( -6.0*(float)w/h, 6.0*(float)w/h, -6.0, 6.0, 1.0, 15.0);
    glMatrixMode(GL_MODELVIEW);
    glViewport(0, 0, w, h);

    glutPostRedisplay();
}

/**
 * Makes Luffy walk.  This function only changes the angles of his joints, a
 * call to glutPostRedisplay() must be done to see the changes.
 */
void walk()
{
    // initialize
    rtAx = ltAx = 1;
    rtAy = ltAy = 0;
    rtAz = ltAz = 0;

    if (ltAng > 30) {
        // time to move the leg backwards
        isForward = false;
    }
    else if (ltAng < -30) {
        isForward = true;
    }

    ltAng += isForward ? 1 : -1;
    rtAng += isForward ? -1 : 1;
}

/**
 * One possible Luffy action.  Makes him jump in the air with feet and hands
 * in the air.
 */
void jump()
{
    // initialize some angles
    ltAx = ltAy = rtAx = rtAy = 0;
    rtAz = ltAz = 1;

    llAng = 0;
    rlAng = 0;
    ulAng = -30;
    urAng = 30;
    lfAng = 80;
    rfAng = -80;

    if (luffyY > 2)
        isFall = true;

    if (!isFall) {
        // jumping
        luffyY += 0.05;
        ltAng -= 1;
        rtAng += 1;
    }
    else if (isFall && luffyY > 0) {
        // falling
        luffyY -= 0.05;
        ltAng += 1;
        rtAng -= 1;
    }

    if (luffyY <= 0)
        isFall = false;
}

/**
 * One possible Luffy action.  Makes him wave with his right hand.
 */
void wave()
{
    // initialize angles
    urAng = 0;
    luffyX = luffyY = luffyZ = 0;

    if (rfAng < -30) {
        isRight = false;
    }
    else if (rfAng > 30) {
        isRight = true;
    }

    rfAng += isRight ? -1.5 : 1.5;

    //printf("rfAng is %f\n", rfAng);
}

/**
 * Forces Luffy into the crouching tiger stance.
 */
void tiger()
{
    const float atkRate = 3;

    ltAng = -60;
    rtAng = 60;
    ltAx = ltAy = rtAx = rtAy = 0;
    ltAz = rtAz = 1;

    llAng = 60;
    rlAng = -60;

    lfAx = rfAx = 1;
    lfAy = lfAz = rfAy = rfAz = 0;

    if (lfAng > 50)
        isAttacking = false;
    else if (lfAng < 0)
        isAttacking = true;

    lfAng += isAttacking ? atkRate : -atkRate;
    rfAng += isAttacking ? atkRate : -atkRate;
}

/**
 * Luffy flaps his arms about like a chicken.  Modifies the necessary angles.
 * A call to glutPostRedisplay() must be done to see the effect.
 */
void chickenDance()
{
    const float flapRate = 2;

    rfAx = rfAy = lfAx = lfAy = 0;
    rfAz = lfAz = 1;
	rfAng = 60;
	lfAng = -60;

	if (ulAng >= 40)
		isFlapping = true;

	if (ulAng <= -20)
		isFlapping = false;

    ulAng += isFlapping ? -flapRate : flapRate;
    urAng += isFlapping ? flapRate : -flapRate;
}

/**
 * Performs the action corresponding to the given value.  This function should
 * only be called from the glut menu.
 */
void processMenu(int val)
{
    switch(val) {
        case NOTHING:
            mode = NOTHING;
            break;
        case WAVE:
            mode = WAVE;
            break;
        case JUMP:
            mode = JUMP;
            break;
        case WALK:
            mode = WALK;
            break;
        case TIGER:
            mode = TIGER;
            break;
        case CHICKEN:
            mode = CHICKEN;
            break;
        case EXIT:
            exit(EXIT_SUCCESS);
    }
}

/**
 * Creates the glut menu that allows the users to select a different action
 * for Luffy to perform.
 */
void createMenu()
{
    glutCreateMenu(processMenu);
    glutAddMenuEntry("Do nothing", NOTHING);
    glutAddMenuEntry("Wave", WAVE);
    glutAddMenuEntry("Jump", JUMP);
    glutAddMenuEntry("Walk", WALK);
    glutAddMenuEntry("Crouching Tiger", TIGER);
    glutAddMenuEntry("Chicken Dance", CHICKEN);
    glutAddMenuEntry("Exit", EXIT);
    glutAttachMenu(GLUT_RIGHT_BUTTON);
}

int main(int argc, char* argv[])
{
    //set up my window
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
    glutInitWindowSize(500, 500); 
    glutInitWindowPosition(100, 100);
    glutCreateWindow("Luffy");
    glClearColor(1.0, 1.0, 1.0, 1.0);

    // Callbacks
    glutDisplayFunc( display );
	glutIdleFunc(idle);
    glutKeyboardFunc(keyboard);
    glutReshapeFunc( reshape );
    glutSpecialFunc(processSpecialKeys);

    // Initializations
    glEnable(GL_DEPTH_TEST);
    max_x = max_y = max_z = FLT_MIN;
    min_x = min_y = min_z = FLT_MAX;
    center.x = 0;
    center.y = 0;
    center.z = 0;
    max_extent = 1.0;
    Initialize();
    createMenu();

    //make sure a file to read is specified
    if (argc > 1) {
        cout << "file " << argv[1] << endl;
        //read-in the mesh file specified
        ReadFile(argv[1]);
        //only for debugging
        if (Vertices.size() > 4)
            printFirstThree();

        //once the file is parsed find out the maximum extent to center and scale mesh
        max_extent = max_x - min_x;
        if (max_y - min_y > max_extent) max_extent = max_y - min_y;
        //cout << "max_extent " << max_extent << " max_x " << max_x << " min_x " << min_x << endl;
        //cout << "max_y " << max_y << " min_y " << min_y << " max_z " << max_z << " min_z " << min_z << endl;

        center.x = center.x/Vertices.size();
        center.y = center.y/Vertices.size();
        center.z = center.z/Vertices.size();
        //cout << "center " << center.x << " " << center.y << " " << center.z << endl;
        //cout << "scale by " << 1.0/(float)max_extent << endl;
    } else {
        cout << "format is: meshparser filename" << endl;
    }

    glutMainLoop();
}

