#include "Vector.h"

Vector::Vector(float pxIn, float pyIn, float pzIn, float xIn, float yIn, float 
  zIn) : px(pxIn), py(pyIn), pz(pzIn), x(xIn), y(yIn), z(zIn)
{
    length = sqrtf(x*x + y*y + z*z);
}

Vector::Vector(float xIn, float yIn, float zIn) :
  px(0), py(0), pz(0), x(xIn), y(yIn), z(zIn)
{
    length = sqrtf(x*x + y*y + z*z);
}

Vector::Vector(float pxIn, float pyIn, float xIn, float yIn) : px(pxIn),
  py(pyIn), pz(0), x(xIn), y(yIn), z(0)
{
	length = sqrt(x*x + y*y);
}

Vector::Vector(float pxIn, float xIn) : px(pxIn), py(0), pz(0), x(xIn), y(0),
  z(0) 
{
}

/* Computes the dot product of the two given Vectors a and b.
 * Returns the result as a float.
 */
float Vector::dotProduct(Vector a, Vector b)
{
	return a.x*b.x + a.y*b.y + a.z*b.z;
}

Vector Vector::crossProduct(Vector a, Vector b)
{
    return Vector(0, 0, 0, a.y*b.z - a.z*b.y, 
                           a.z*b.x - a.x*b.z,
                           a.x*b.y - a.y*b.x);
}

Vector Vector::operator+(Vector rhs)
{
    float x = this->x + rhs.x;
    float y = this->y + rhs.y;
    float z = this->z + rhs.z;

    return Vector(x, y, z);
}

Vector Vector::operator+=(Vector rhs)
{
    *this = *this + rhs;

    return *this;
}

Vector Vector::operator*(float rhs)
{
    float x = this->x*rhs;
    float y = this->y*rhs;
    float z = this->z*rhs;

    return Vector(x, y, z);
}

Vector Vector::operator/(float rhs)
{
    float x = this->x/rhs;
    float y = this->y/rhs;
    float z = this->z/rhs;

    return Vector(x, y, z);
}

Vector Vector::operator=(Vector rhs)
{
    this->x = rhs.x;
    this->y = rhs.y;
    this->z = rhs.z;
    this->px = rhs.px;
    this->py = rhs.py;
    this->pz = rhs.pz;
    this->length = rhs.length;

    return rhs;
}

float Vector::getLength()
{
	return length;
}

Vector Vector::computePerpendicular()
{
    float midX = (2*px+x)/2.0, midY = (y+2*py)/2.0;

    return Vector(midX, midY, -y, x);
}

void Vector::plot()
{
	// Ending coordinates
	float ex = px+0.01*x;
	float ey = py+0.01*y;
	float ez = pz+0.01*z;

	glBegin(GL_LINES);
		glVertex3f(px, py, pz);
		glVertex3f(ex, ey, ez);
	glEnd();

	glPointSize(5);
	glBegin(GL_POINTS);
		glVertex3f(ex, ey, ez);
	glEnd();
}

Vector Vector::normalize()
{
	x = x/length;
	y = y/length;
	z = z/length;

	length = sqrtf(x*x + y*y + z*z);

    return *this;
}

float* Vector::toArray()
{
    float* returnVal = new float[3];
    returnVal[0] = x;
    returnVal[1] = y;
    returnVal[2] = z;

    return returnVal;
}